# NeuralFieldEq.jl


### Table of Contents

```@contents
Pages = ["index.md", "usage.md", "examples.md", "apidocs.md"]
Depth = 2
```

!!! note

    This package is intended for both experienced researchers and graduate students who are taking their first steps, in this last case is recommended that the user is familiarised with the basics of Neural Field Equations (NFE). See [Amari 1977](https://bsi-ni.brain.riken.jp/database/file/64/050_2.pdf) and [Bressloff 2011](https://www.math.utah.edu/~bresslof/publications/11-7.pdf).

## About

The numerical method implemented in `NeuralFieldEq.jl` was developed within the scope of the master's thesis [Sequeira 2021](https://fenix.tecnico.ulisboa.pt/cursos/mma/dissertacao/1691203502344856) under the supervision of professor Pedro M. Lima. The method combined the novel numerical scheme published originally by [Hutt & Rougier 2013](https://hal.inria.fr/hal-00872132/document) for delayed NFE in the context of the stochastic scenario presented by [Kuehn & Riedler 2014](https://link.springer.com/content/pdf/10.1186/2190-8567-4-1.pdf) where the convergence of spectral methods was proved in stochastic neural fields with additive white noise and spatial correlation.

Hence, this package aims to numerically approximate solutions of Neural Field Equations in one- or two-dimensional spaces, with or without delay and in the deterministic or stochastic scenarios described below.

## Introducing Neural Field Equations

Due to the large concentration of neurons in the cerebral cortex (``\approx 10^{10}``) distributed along ``2500\,cm^2`` of surface area with ``2.8\,mm`` of depth, this layer of the brain can be viewed as a virtual 2D continuum space. The NFE arose from the necessity of modelling the neuronal activity present in cortical tissues. Amari derived the following equation
```math
\alpha \frac{\partial V}{\partial t}\left(\mathbf{x},t\right) = I\left(\mathbf{x},t\right) - V\left(\mathbf{x},t\right) + \int_{\Omega} K\left(||\mathbf{x}-\mathbf{y}||_2\right)S\big[V\left(\mathbf{y},t\right)\big]\,\,d^2\mathbf{y},
```
where ``\Omega=[-\frac{L}{2},\frac{L}{2}]^d``, with ``d=1,2``; ``V(\mathbf{x},t)`` is the membrane potential of a neuron located at ``\mathbf{x} \in \Omega`` at instant ``t``; ``I(\mathbf{x},t)`` is the external input applied to the neural field; ``K\left(||\mathbf{x}-\mathbf{y}||_2\right)`` is the average strength of connectivity between neurons located at points ``\mathbf{x}`` and ``\mathbf{y}``. ``K`` comprises the type of the synapse, when the coupling is positive (negative) the synapses are excitatory (inhibitory); ``S(V)`` is the firing rate function, a monotonically non-decreasing function that converts the potential to the respective firing rate result; And ``\alpha`` is the constant decay rate.

## NFE with finite transmission speed (delay)

The equation written above do not consider the finite velocity propagation of the action potentials, depending on the cortical region this assumption can be biological inaccurate since the axonal speed can vary between ``100\,m/s`` to ``1\,m/s``. Thus, in order to work with a more realistic model the neural field equation can be written taking into consideration the time spent by the stimulus to travel from neurons at ``y`` to the ones at ``x``:
```math
\alpha \frac{\partial V}{\partial t}\left(\mathbf{x},t\right) = I\left(\mathbf{x},t\right) - V\left(\mathbf{x},t\right) + \int_{\Omega} K\left(||\mathbf{x}-\mathbf{y}||_2\right)S\big[V\left(\mathbf{y},t-d\left(\mathbf{x},\mathbf{y}\right)\right)\big]\,\,d^2\mathbf{y},
```
with ``d=\frac{||\mathbf{x}-\mathbf{y}||_2}{v}`` representing the delay, which is assumed to only depend on the distance and on the transmission speed ``v``.
**Remark**: If ``v`` is sufficiently high, ``d`` can be neglected and the last equation is reduced to the non-delayed equation.

## Sotchastic NFE with delay

One of the many endeavours to, iteratively, improve Neural Field models is to considering the noisy neuronal interactions, generated by the stochastic behaviour of the neuron or from extrinsic noise sources that can arise from inputs of other neuronal networks. [Kuehn & Riedler 2014](https://link.springer.com/content/pdf/10.1186/2190-8567-4-1.pdf) studied a neural field model with additive white noise and ``v=\infty``. If we add the delay function presented above we end up with the following model:
```math
\alpha\, dV\left(\mathbf{x},t\right) = \left[I\left(\mathbf{x},t\right) - V\left(\mathbf{x},t\right) + \int_{\Omega}K\left(||\mathbf{x}-\mathbf{y}||_2\right)S\big[V\left(\mathbf{y},t-d\left(\mathbf{x},\mathbf{y}\right)\right)\big]\,\,d^2\mathbf{y}\right]dt + \epsilon dW\left(\mathbf{x},t\right),
```
with ``\epsilon`` the additive noise level and ``W`` is a ``Q``-Wiener process.

## References

Amari, Shun-ichi. (1977). Dynamic of pattern formation in lateral-inhibition type neural fields. Biological cybernetics. 27. 77-87. 10.1007/BF00337259.

Bressloff, Paul. (2011). Spatiotemporal dynamics of continuum neural fields. Journal of Physics A: Mathematical and Theoretical. 45. 033001. 10.1088/1751-8113/45/3/033001.

Hutt, Axel & Rougier, Nicolas. (2013). Numerical Simulation Scheme of One- and Two Dimensional Neural Fields Involving Space-Dependent Delays. 10.1007/978-3-642-54593-1_6.

Laing, Carlo & Troy, William & Gutkin, Boris & Ermentrout, Bard. (2002). Multiple Bumps in a Neuronal Model of Working Memory. SIAM Journal on Applied Mathematics. 63. 10.1137/S0036139901389495. 

Kuehn, Christian & Riedler, Martin. (2014). Large Deviations for Nonlocal Stochastic Neural Fields. Journal of mathematical neuroscience. 4. 1. 10.1186/2190-8567-4-1.

Kulikov, Gennady & Kulikova, Maria & Lima, Pedro. (2019). Numerical Simulation of Neural Fields with Finite Transmission Speed and Random Disturbance. 644-649. 10.1109/ICSTCC.2019.8885972.

Kulikova, Maria & Kulikov, Gennady & Lima, Pedro. (2019). Effective Numerical Solution to Two-Dimensional Stochastic Neural Field Equations. 650-655. 10.1109/ICSTCC.2019.8885614. 

Sequeira, Tiago (2021). Numerical Simulations of One- and Two-dimensional Stochastic Neural Field Equations with Delay - MSc in Industrial Mathematics dissertation.